@@ -40,6 +40,7 @@ my $INCORRECT_LOCALE_CODES
 my $MULTIARCH_DIRS = Lintian::Data->new('common/multiarch-dirs', qr/\s++/);
 my $GENERIC_HEADER_FILES = Lintian::Data->new('files/generic-header-files');
 my $GENERIC_PYTHON_MODULES= Lintian::Data->new('files/generic-python-modules');
+my $ALLOWED_ANCIENT_FILES = Lintian::Data->new('files/allowed-ancient-files');
 
 my $PRIVACY_BREAKER_WEBSITES= Lintian::Data->new(
     'files/privacy-breaker-websites',
@@ -94,6 +95,10 @@ my $PKG_CONFIG_BAD_REGEX
   = Lintian::Data->new('files/pkg-config-bad-regex',qr/~~~~~/,
     sub { return  qr/$_[0]/xsm;});
 
+my $DOCUMENTATION_FILE_REGEX
+  = Lintian::Data->new('files/documentation-file-regex',
+    qr/~~~~~/,sub { return  qr/$_[0]/xi;});
+
 my $COMPRESS_FILE_EXTENSIONS
   = Lintian::Data->new('files/compressed-file-extensions',
     qr/\s++/,sub { return qr/\Q$_[0]\E/ });
@@ -164,7 +169,7 @@ sub _load_file_package_list_mapping {
 }
 
 my $JS_EXT
-  = '(?:(?i)[-._]?(?:compiled|lite|min|pack(?:ed)?|prod|umd|yc)?\.js(?:\.gz)?)$';
+  = '(?:(?i)[-._]?(?:compiled|lite|min|pack(?:ed)?|prod|umd|yc)?\.(js|css)(?:\.gz)?)$';
 my $PHP_EXT = '(?i)\.(?:php|inc|dtd)$';
 my @FILE_PACKAGE_MAPPING = (
     _load_file_package_list_mapping(
@@ -337,6 +342,8 @@ sub run {
     # Check if package is empty
     my $is_dummy = $info->is_pkg_class('any-meta');
 
+    my $has_sensible_utils = $info->relation('all')->implies('sensible-utils');
+
     # read data from objdump-info file
     foreach my $file (sort keys %{$info->objdump_info}) {
         my $objdump = $info->objdump_info->{$file};
@@ -445,9 +452,9 @@ sub run {
         }
 
         my ($year) = ($file->date =~ /^(\d{4})/);
-        if ($year <= 1975) { # value from dak CVS: Dinstall::PastCutOffYear
-            tag 'package-contains-ancient-file', $file, $file->date;
-        }
+        tag 'package-contains-ancient-file', $file, $file->date
+          if $year <= 1975 # value from dak CVS: Dinstall::PastCutOffYear
+          and not $ALLOWED_ANCIENT_FILES->matches_any($fname);
 
         if (
             !(
@@ -458,7 +465,7 @@ sub run {
                    $file->gid < 100
                 || $file->gid == 65_534
                 || ($file->gid >= 60_000 && $file->gid < 65_000))
-          ) {
+        ) {
             tag 'wrong-file-owner-uid-or-gid', $file,
               $file->uid . '/' . $file->gid;
         }
@@ -492,7 +499,7 @@ sub run {
             if(index($fname,'oui') > -1 || index($fname,'iab') > -1) {
                 if($fname
                     =~ m,/(?:[^/]-)?(?:oui|iab)(?:\.(txt|idx|db))?(?:\.$COMPRESS_FILE_EXTENSIONS_OR_ALL)?\Z,x
-                  ) {
+                ) {
                     unless ($source_pkg eq 'ieee-data') {
                         tag 'package-installs-ieee-data', $file;
                     }
@@ -517,15 +524,23 @@ sub run {
                 }
                 # -----------------/etc/apt/sources
                 if ($fname =~ m,^etc/apt/sources\.list(?:$|\.d/[^/]+),) {
-                    unless ($source_pkg eq 'apt') {
+                    unless ($source_pkg eq 'apt'
+                        or $pkg =~ /-apt-source$/) {
                         tag 'package-installs-apt-sources', $file;
                     }
                 }
+                # -----------------/etc/apt/trusted.gpg
+                if ($fname =~ m,^etc/apt/trusted\.gpg(?:$|\.d/[^/]+),) {
+                    unless ($source_pkg eq 'apt'
+                        or $pkg =~ /(?:-apt-source|-archive-keyring)$/) {
+                        tag 'package-installs-apt-keyring', $file;
+                    }
+                }
             }
             # ---------------- /etc/cron.daily, etc.
             elsif ($fname
                 =~ m,^etc/cron\.(?:daily|hourly|monthly|weekly|d)/[^\.].*[\+\.],
-              ) {
+            ) {
                 # NB: cron ships ".placeholder" files, which shouldn't be run.
                 tag 'run-parts-cron-filename-contains-illegal-chars', $file;
             }
@@ -609,12 +624,10 @@ sub run {
         elsif ($fname =~ m,^usr/,) {
             # ---------------- /usr/include
             if ($fname =~ m,^usr/include/,) {
-                if ($file->is_file and $header_dirs{$file->dirname}) {
-                    for my $regex ($GENERIC_HEADER_FILES->all) {
-                        tag 'header-has-overly-generic-name', $fname,
-                          if $file->basename =~ m,$regex,i;
-                    }
-                }
+                tag 'header-has-overly-generic-name', $fname
+                  if $file->is_file
+                  and $header_dirs{$file->dirname}
+                  and $GENERIC_HEADER_FILES->matches_any($file->basename, 'i');
                 # ---------------- /usr/share/doc
             } elsif ($fname =~ m,^usr/share/doc/\S,) {
                 if ($type eq 'udeb') {
@@ -675,9 +688,10 @@ sub run {
 
                     # contains an INSTALL file?
                     if ($fname =~ m,^usr/share/doc/$ppkg/INSTALL(?:\..+)*$,){
-                        tag
-                          'package-contains-upstream-install-documentation',
+                        #<<< No perltidy - tag name too long
+                        tag 'package-contains-upstream-installation-documentation',
                           $file;
+                        #>>>
                     }
 
                     # contains a README for another distribution/platform?
@@ -688,7 +702,7 @@ sub run {
                                |macosx|netbsd|openbsd|osf|redhat|sco|sgi
                                |solaris|suse|sun|vms|win32|win9x|windows
                              )(?:\.txt)?(?:\.gz)?$,xi
-                      ) {
+                    ) {
                         #<<< No tidy (tag name too long)
                         tag 'package-contains-readme-for-other-platform-or-distro',
                           $file;
@@ -766,7 +780,7 @@ sub run {
                 if (   $file->is_file
                     && $fname
                     =~ m,^usr/lib/debug/usr/lib/pyshared/(python\d?(?:\.\d+))/(.++)$,o
-                  ) {
+                ) {
                     my $correct = "usr/lib/debug/usr/lib/pymodules/$1/$2";
                     tag 'python-debug-in-wrong-location', $file, $correct;
                 }
@@ -881,7 +895,7 @@ sub run {
                 && $fname !~ m{\A usr/share/p11-kit/modules/
                                   [[:alnum:]][[:alnum:]_.-]*\.module\Z
                               }xsm
-              ) {
+            ) {
                 tag 'incorrect-naming-of-pkcs11-module', $file;
             }
             # ---------------- /usr/share/vim
@@ -920,7 +934,7 @@ sub run {
                         and (  index($block,'flag')>-1
                             or index($block,'/include/') > -1
                             or index($block,'pkg-config')  > -1)
-                      ) {
+                    ) {
                         tag 'old-style-config-script',$file;
                         # could be ok but only if multi-arch: no
                         if($multiarch ne 'no' or $arch eq 'all') {
@@ -970,7 +984,7 @@ sub run {
                                     lib|
                                     local|sbin|share|
                                     src|spool|tmp)/,x
-                  ) {
+                ) {
                     if ($fname =~ m,^usr/lib(?'libsuffix'64|x?32)/,) {
                         my $libsuffix = $+{libsuffix};
                         # eglibc exception is due to FHS. Other are
@@ -984,7 +998,8 @@ sub run {
                             tag 'non-multi-arch-lib-dir', $file;
                         }
                     } else {
-                        tag 'non-standard-dir-in-usr', $file;
+                        tag 'non-standard-dir-in-usr', $file
+                          unless $fname =~ m,^usr/libexec/,; # #834607
                     }
 
                 }
@@ -1035,7 +1050,7 @@ sub run {
                                 |lock|log|opt|run|spool|state
                                 |tmp|www|yp)/
              }xsm
-              ) {
+            ) {
 
                 tag 'non-standard-dir-in-var', $file;
             }
@@ -1096,7 +1111,7 @@ sub run {
                  |mnt|opt|root|run|sbin|srv|sys
                  |tmp|usr|var)  /
           }oxsm
-          ) {
+        ) {
             # Make an exception for the base-files package here and
             # other similar packages because they install a slew of
             # top-level directories for setting up the base system.
@@ -1139,12 +1154,13 @@ sub run {
                 or $fname =~ m,^(?:bin|boot|dev|etc|sbin)/,
                 # non-FHS, but still usual
                 or $fname =~ m,^usr/[^/]+-linux[^/]*/,
+                or $fname =~ m,^usr/libexec/, # FHS 3.0 / #834607
                 or $fname =~ m,^usr/iraf/,
                 # not allowed, but tested individually
                 or $fname =~ m{\A (?:
                         build|home|mnt|opt|root|run|srv
                        |(?:(?:usr|var)/)?tmp)|var/www/}xsm
-              ) {
+            ) {
                 tag 'file-in-unusual-dir', $file;
             }
 
@@ -1193,7 +1209,7 @@ sub run {
             && (   $fname =~ m,^usr/lib/python\d+(?:\.\d+/),o
                 || $fname =~ m,^usr/lib/pyshared,o
                 || $fname =~ m,^usr/share/,o)
-          ) {
+        ) {
             tag 'package-installs-python-egg', $file;
         }
 
@@ -1260,7 +1276,7 @@ sub run {
                           # ... so they end with ".pl" rather than ".pm"
                           \.pl['"]
                }xsm
-                      ) {
+                    ) {
                         tag 'perl-module-uses-perl4-libs-without-dep',
                           "$file:$. ${1}.pl";
                     }
@@ -1326,7 +1342,7 @@ sub run {
             and not defined $link
             # Sphinx includes various license files
             and not $fname =~ m,/_sources/license(\.rst)?\.txt$,oi
-          ) {
+        ) {
 
             # okay, we cannot rule it out based on file name; but if
             # it is an elf or a static library, we also skip it.  (In
@@ -1344,7 +1360,7 @@ sub run {
             # check later to see if it's in a symlinked directory.
             and not $fname =~ m,^usr/share/(?:devhelp/books|gtk-doc/html)/,
             and not $fname =~ m,^usr/share/doc/[^/]+/examples/,
-          ) {
+        ) {
             push(@devhelp, $fname);
         }
 
@@ -1378,7 +1394,7 @@ sub run {
         if (
             $fname =~ m,^usr/share/doc/(?:.+/)?(?:doxygen|html)/
                          .*\.map\.$COMPRESS_FILE_EXTENSIONS_OR_ALL,x
-          ) {
+        ) {
             tag 'file-should-not-be-compressed', $file;
         }
 
@@ -1401,7 +1417,7 @@ sub run {
                   usr/lib/python (\d+(?:\.\d+)?)/
                         (site|dist)-packages/(.++)
         \Z}oxsm
-          ){
+        ){
             my ($debug, $pyver, $loc, $rest) = ($1, $2, $3, $4);
             my ($pmaj, $pmin) = split(m/\./o, $pyver, 2);
             my @correction;
@@ -1473,6 +1489,32 @@ sub run {
                 tag 'nfs-temporary-file-in-package', $file;
             }
 
+            # ---------------- using sensible-utils w/o dependency
+            tag 'missing-depends-on-sensible-utils', $file
+              if not $has_sensible_utils
+              and $fname !~ m,^usr/share/(?:doc|locale)/,
+              and $source_pkg ne 'lintian'
+              and detect_sensible_utils($file);
+
+            # ---------------- documentation files
+            unless($fname =~ m,^etc/, or $fname =~ m,^usr/share/doc/,) {
+                foreach my $taboo ($DOCUMENTATION_FILE_REGEX->all) {
+                    my $regex = $DOCUMENTATION_FILE_REGEX->value($taboo);
+                    if($file->basename =~ m{$regex}xi) {
+                        # see #904852
+                        next if $file->dirname =~ m{templates?(?:\.d)?/};
+                        next
+                          if $file->basename =~ m{^README}xi
+                          and $file->file_contents =~ m{this directory}xi;
+                        #<<< No perltidy - tag name too long
+                        tag 'package-contains-documentation-outside-usr-share-doc',
+                          $fname;
+                        #>>>
+                        last;
+                    }
+                }
+            }
+
             # ---------------- vcs control files
             if (    $fname =~ m,$VCS_FILES_OR_ALL,
                 and $fname !~ m,^usr/share/cargo/registry/,) {
@@ -1493,7 +1535,7 @@ sub run {
                            (?:usr/)?(?:s?bin|games)/[^/]+\.
                            (?:p[ly]|php|rb|[bc]?sh|tcl)
                          \Z}xsm
-              ) {
+            ) {
                 tag 'script-with-language-extension', $file;
             }
 
@@ -1535,7 +1577,7 @@ sub run {
             }
 
             # ---------------- html/javascript
-            if ($fname =~ m,\.(?:x?html?|js|xht|xml|css)$,i) {
+            if ($fname =~ m,\.(?:x?html?\d?|js|xht|xml|css)$,i) {
                 if(     $source_pkg eq 'josm'
                     and $file->basename eq 'defaultpresets.xml') {
                     # false positive
@@ -1544,7 +1586,8 @@ sub run {
                 }
             }
             # ---------------- fonts
-            elsif ($fname =~ m,/([\w-]+\.(?:[to]tf|pfb))$,i) {
+            elsif (
+                $fname =~ m,/([\w-]+\.(?:[to]tf|pfb|woff2?|eot)(?:\.gz)?)$,i) {
                 my $font = lc $1;
                 if (my $font_owner = $FONT_PACKAGES->value($font)) {
                     tag 'duplicate-font-file', "$fname also in", $font_owner
@@ -1552,6 +1595,9 @@ sub run {
                 } elsif ($pkg !~ m/^(?:[ot]tf|t1|x?fonts)-/) {
                     tag 'font-in-non-font-package', $file;
                 }
+                tag 'font-outside-font-dir', $file
+                  unless $fname =~ m,^usr/share/fonts/,;
+
                 my $finfo = $file->file_info;
                 if ($finfo =~ m/PostScript Type 1 font program data/) {
                     my $path = $file->fs_path;
@@ -1563,7 +1609,7 @@ sub run {
                                 $line =~ m{\A [%\s]*
                                    All\s*Rights\s*Reserved\.?\s*
                                        \Z}xsmi
-                              ) {
+                            ) {
                                 #<<< No perltidy - tag name too long
                                 tag 'license-problem-font-adobe-copyrighted-fragment',
                                   $file;
@@ -1578,7 +1624,7 @@ sub run {
                                [%\s]*Copyright\s*\(c\) \s*
                                19\d{2}[\-\s]19\d{2}\s*
                                Adobe\s*Systems\s*Incorporated\.?\s*\Z}xsmi
-                          ) {
+                        ) {
                             $foundadobeline = 1;
                         }
                         # If copy pasted from black book they are
@@ -1760,7 +1806,7 @@ sub run {
                 )
                 and $operm == 01777
                 and $owner eq 'root/root'
-              ) {
+            ) {
                 # actually shipping files here is warned about elsewhere
             } elsif ($fname eq 'usr/src/'
                 and $operm == 02775
@@ -1945,6 +1991,20 @@ sub run {
         }
     }
 
+    # Maintainer scripts
+    open(my $fd, '<', $info->lab_data_path('control-scripts'));
+    while (<$fd>) {
+        m/^\S* (.*)$/
+          or internal_error("bad line in control-scripts file: $_");
+
+        my $file = $info->control_index_resolved_path($1);
+        next if not $file or not $file->is_open_ok;
+
+        tag 'missing-depends-on-sensible-utils', $file
+          if not $has_sensible_utils and detect_sensible_utils($file);
+    }
+    close($fd);
+
     if (!$is_dummy && !$arch_dep_files && $arch ne 'all') {
         tag 'package-contains-no-arch-dependent-files'
           unless $type eq 'udeb';
@@ -2072,6 +2132,8 @@ sub _check_tag_url_privacy_breach {
                 'author'            => 1, # #891301
                 'bookmark'          => 1, # #746656
                 'canonical'         => 1, # #762753
+                'copyright'         => 1, # #902919
+                'edituri'           => 1, # #902919
                 'generator'         => 1, # #891301
                 'generator-home'    => 1, # texinfo
                 'help'              => 1, # #891301
@@ -2232,7 +2294,7 @@ sub detect_privacy_breach {
             qw(src="http src="ftp src="// data-href="http data-href="ftp
             data-href="// codebase="http codebase="ftp codebase="// data="http
             data="ftp data="// poster="http poster="ftp poster="// <link @import)
-          ) {
+        ) {
             next if index($block, $x) == -1;
             detect_generic_privacy_breach($block,\%privacybreachhash,$file);
             last;
@@ -2242,6 +2304,20 @@ sub detect_privacy_breach {
     return;
 }
 
+sub detect_sensible_utils {
+    my ($file) = @_;
+
+    my $fd = $file->open(':raw');
+    my $sfd = Lintian::SlidingWindow->new($fd, sub { $_= lc($_); }, BLOCKSIZE);
+
+    while (my $block = $sfd->readwindow) {
+        return 1
+          if $block=~ m/(?:select-editor|sensible-(?:browser|editor|pager))\b/;
+    }
+    close($fd);
+    return;
+}
+
 1;
 
 # Local Variables:
